/**
  **************************************************************************
  * @file     iap.c
  * @version  v2.0.1
  * @date     2022-09-16
  * @brief    iap program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to
  * download from Artery official website is the copyrighted work of Artery.
  * Artery authorizes customers to use, copy, and distribute the BSP
  * software and its related documentation for the purpose of design and
  * development in conjunction with Artery microcontrollers. Use of the
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "iap.h"
#include "usart.h"
#include "flash.h"
#include "tmr.h"

/** @addtogroup UTILITIES_examples
  * @{
  */

/** @addtogroup USART_IAP_bootloader
  * @{
  */

cmd_data_step_type cmd_data_step = CMD_DATA_IDLE;
cmd_ctr_step_type cmd_ctr_step = CMD_CTR_IDLE;
cmd_data_group_type cmd_data_group_struct;
update_status_type update_status = UPDATE_PRE;
iapfun jump_to_app;
uint16_t jump_flag = 0;

extern uint16_t end_cnt;
extern uint32_t timeout;
extern uint32_t bin_cnt;
extern uint16_t data_length;
uint8_t send_buff[2048];

/* app_load don't optimize */
#if defined (__CC_ARM)
  #pragma O0
#elif defined (__ICCARM__)
  #pragma optimize=s none
#endif
/**
  * @brief  app load.
  * @param  app_addr
  *         app code starting address
  * @retval none
  */
void app_load(uint32_t app_addr)
{
  /* check the address of stack */
  if(((*(uint32_t*)app_addr) - 0x20000000) < (SRAM_SIZE * 1024))
  {
    /* disable periph clock */
    crm_periph_clock_enable(CRM_TMR3_PERIPH_CLOCK, FALSE);
    crm_periph_clock_enable(CRM_USART3_PERIPH_CLOCK, FALSE);
    crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, FALSE);

    /* disable nvic irq and clear pending */
    nvic_irq_disable(USART3_IRQn);
    nvic_irq_disable(TMR3_GLOBAL_IRQn);
    __NVIC_ClearPendingIRQ(USART3_IRQn);
    __NVIC_ClearPendingIRQ(TMR3_GLOBAL_IRQn);

    jump_to_app = (iapfun)*(uint32_t*)(app_addr + 4);        /* code second word is reset address */
    __set_MSP(*(uint32_t*)app_addr);                        /* init app stack pointer(code first word is stack address) */
    jump_to_app();                                          /* jump to user app */
  }
	else
  {
    jump_flag = 1;
		printf("jump to app fail\r\n");
	}
}

/**
  * @brief  take data from usart buf.
  * @param  app_addr
  *         app code starting address
  * @retval val
  *         took data
  */
uint8_t data_take(void)
{
  uint8_t val;
  usart_interrupt_enable(USART3, USART_RDBF_INT, FALSE);
  usart_group_struct.count--;
  usart_interrupt_enable(USART3, USART_RDBF_INT, TRUE);
  val = usart_group_struct.buf[usart_group_struct.tail++];
  if(usart_group_struct.tail > (USART_REC_LEN-1))
    usart_group_struct.tail = 0;
  return val;
}

void update_handle(void)
{
  static uint8_t buff_cnt_flag = 0;
  
  if(usart_group_struct.buff_cnt > 0)
  {
    timeout++;
    if(timeout > 0x7fffff)
    {
      timeout = 0;
			printf("Receiving OTA data timeout..\r\n\r\n");

      clear_ota_data();
      nvic_system_reset();
    }
  }
  if(usart_group_struct.buff_cnt != buff_cnt_flag)
  {
    buff_cnt_flag = usart_group_struct.buff_cnt;
    switch(usart_group_struct.buff_cnt%2)
    {
      case 1:
          flash_2kb_write(FLASH_OTA_ADDR + (2048*(usart_group_struct.buff_cnt-1)), usart_group_struct.buf);
        break;
      case 0:
          flash_2kb_write(FLASH_OTA_ADDR + (2048*(usart_group_struct.buff_cnt-1)), usart_group_struct.buf+2048);
        break;
      default:
        break;
    }  
  }
  if(end_cnt == 4)
  {
    end_cnt = 0;
    if(usart_group_struct.head < 2048)
    {
      flash_nbyte_write(FLASH_OTA_ADDR + (2048*(usart_group_struct.buff_cnt)), usart_group_struct.buf, usart_group_struct.count-4);
    }
    else
    {
      flash_nbyte_write(FLASH_OTA_ADDR + (2048*(usart_group_struct.buff_cnt)), usart_group_struct.buf+2048, usart_group_struct.count-4);
    }
		if((bin_cnt-4) == data_length)
		{
			printf("\r\nReceived OTA data ok!!\r\n");
			update_status = UPDATE_DONE;
      back_ok();
		}
    else
    {
      back_err();
			printf("\r\nReceived OTA data error!!\r\n");
      
      clear_ota_data();
      nvic_system_reset();
    }
  }
}

/**
  * @brief  command response ok.
  * @param  none
  * @retval none
  */
void back_ok(void)
{
  usart_data_transmit(USART3, 0xCC);
  while(usart_flag_get(USART3, USART_TDC_FLAG) == RESET);
  usart_data_transmit(USART3, 0xDD);
  while(usart_flag_get(USART3, USART_TDC_FLAG) == RESET);
}

/**
  * @brief  command response error.
  * @param  none
  * @retval none
  */
void back_err(void)
{
  usart_data_transmit(USART3, 0xCC);
  while(usart_flag_get(USART3, USART_TDC_FLAG) == RESET);
  usart_data_transmit(USART3, 0xEE);
  while(usart_flag_get(USART3, USART_TDC_FLAG) == RESET);
  /* reset state machine of udgrade flow */
  cmd_data_step = CMD_DATA_IDLE;
  cmd_ctr_step = CMD_CTR_IDLE;
  update_status = UPDATE_PRE;
  time_ira_cnt = 0;
  get_data_from_usart_flag = 0;
}

/**
  * @brief  app update flow handle.
  * @param  none
  * @retval none
  */
uint8_t buffer_read[2048];
void app_update_handle(void)
{
  uint16_t i;
    if(update_status == UPDATE_DONE)
    {
      for(i = 0; i < APP_SIZE; i++)
      {
        flash_unlock();
        flash_sector_erase(APP_START_ADDR + i*2048);
        flash_lock();
        flash_read((FLASH_OTA_ADDR+0x10) + i*2048, buffer_read, 2048);
        flash_2kb_write(APP_START_ADDR + (i*2048), buffer_read);
      }
      
			printf("flash write done\r\n\r\n");
      update_status = UPDATE_PRE;
      crm_reset();
      
      clear_ota_data();
      
      /* jump and run in app */
      app_load(APP_START_ADDR);
    }
}

/**
  * @brief  iap upgrade app handle.
  * @param  none
  * @retval none
  */
void clear_ota_data(void)
{
  uint8_t i;
  flash_unlock();
  flash_sector_erase(IAP_UPGRADE_FLAG_ADDR);
  for(i = 0; i < 60; i++)  //64*2048=128k
  {
    flash_sector_erase(FLASH_OTA_ADDR + i*2048);
  }
  flash_lock();
}

/**
  * @brief  iap upgrade app handle.
  * @param  none
  * @retval none
  */
void iap_upgrade_app_handle(void)
{
  update_handle();
  app_update_handle();
}

/**
  * @}
  */

/**
  * @}
  */
